﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WebCrawling.Data.Extracting
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Net;
    using System.Threading;
    using System.Text.RegularExpressions;

    /// <summary>
    /// Google PageRank checker. 
    /// </summary>
    public class GoogleService
    {
        public List<GooglePageRank> Results { get; private set; }

        public GoogleService()
        {

        }

        public GoogleService(IEnumerable<string> urls)
        {
            foreach (var url in urls)
            {
                try
                {
                    this.Add(this.GetPageRank(url));
                }
                catch (Exception ex)
                {
                    this.Add(new GooglePageRank { PageRank = "n/a", Url = url });
                }
            }
        }

        public GooglePageRank GetPageRank(string url)
        {
            //Calculate URL checksum
            var checkSum = this.GoogleChecksum("info:" + url);
            var file = "http://toolbarqueries.google.com/tbr?features=Rank&sourceid=navclient-ff&client=navclient-auto-ff&ch=" + checkSum + "&q=info:" + url;

            //Request PR from Google
            var webClient = new WebClient();
            string data = webClient.DownloadString(new Uri(file));

            //Parse PR from string
            int pageRank = -1;
            if (data.IndexOf(':') != -1)
            {
                data = data.Substring(data.LastIndexOf(':') + 1);
            }

            int.TryParse(data, out pageRank);

            return new GooglePageRank { PageRank = pageRank.ToString(), Url = url };
        }

        public static int GetLinks(string searchUrl, Rule rule)
        {
            int count = 0;
            searchUrl = "link:" + searchUrl;
            String html = Search(rule.Charset, searchUrl, 0);
            string linkCount = html.GetMatchValue(@"<div id=""resultStats"">(.+?)</div>");
            linkCount = Regex.Match(linkCount.Replace(",", ""), @"\d+").Value;
            if (linkCount.IsNotNullOrEmpty())
            {
                count = Int16.Parse(linkCount);
            }
            return count;
        }

        public static List<string> GetSearchUrls(Rule rule)
        {
            List<string> linkUrls = new List<string>();
            //int start = 0;
            //foreach (string UrlPrefix in rule.UrlPrefixs)
            //{
            //    while (true)
            //    {
            //        String html = Search(rule.Charset, "site:" + UrlPrefix, start);
            //        List<string> urls = new List<string>();
            //        urls = html.GetMatchGroups(@"<cite>(.+?)</cite>");
            //        if (urls.Count == 0)
            //        {
            //            break;
            //        }
            //        else
            //        {
            //            foreach (string link in urls)
            //            {
            //                if (link.IsMatch(rule.LinksRegex))
            //                {
            //                    linkUrls.Add(link);
            //                }
            //            }
            //            start = +10;
            //        }

            //    }
            //}
            return linkUrls;
        }

        private static string Search(string charset, string searchKey, int startPage)
        {
            String url = string.Format("https://www.google.com/search?hl={0}&safe=strict&site=&source=hp&q={1}&oq={1}&start={2}", charset, searchKey, startPage);
            PageClass page = HttpRequestSyn.Instance.GetPage(new Uri(url), charset);
            Thread.Sleep(2000);
            return page.html;
        }

        private void Mix(ref uint a, ref uint b, ref uint c)
        {
            a -= b;
            a -= c;
            a ^= c >> 13;
            b -= c;
            b -= a;
            b ^= a << 8;
            c -= a;
            c -= b;
            c ^= b >> 13;
            a -= b;
            a -= c;
            a ^= c >> 12;
            b -= c;
            b -= a;
            b ^= a << 16;
            c -= a;
            c -= b;
            c ^= b >> 5;
            a -= b;
            a -= c;
            a ^= c >> 3;
            b -= c;
            b -= a;
            b ^= a << 10;
            c -= a;
            c -= b;
            c ^= b >> 15;
        }

        private void Add(GooglePageRank pageRank)
        {
            this.Results.Add(pageRank);
        }

        private string GoogleChecksum(string url)
        {
            this.Results = new List<GooglePageRank>();
            uint GoogleMagic = 0xE6359A60;

            uint a, b;
            uint c = GoogleMagic;

            a = b = 0x9E3779B9;

            int k = 0;
            int length = url.Length;

            //Algorithm
            while (length >= 12)
            {
                a += (uint)(url[k + 0] + (url[k + 1] << 8) + (url[k + 2] << 16) + (url[k + 3] << 24));
                b += (uint)(url[k + 4] + (url[k + 5] << 8) + (url[k + 6] << 16) + (url[k + 7] << 24));
                c += (uint)(url[k + 8] + (url[k + 9] << 8) + (url[k + 10] << 16) + (url[k + 11] << 24));

                Mix(ref a, ref b, ref c);

                k += 12;
                length -= 12;
            }

            c += (uint)url.Length;

            //All cases fall through
            switch (length)
            {
                case 11:
                    c += (uint)(url[k + 10] << 24);
                    goto case 10; //fall through
                case 10:
                    c += (uint)(url[k + 9] << 16);
                    goto case 9;
                case 9:
                    c += (uint)(url[k + 8] << 8);
                    goto case 8;
                case 8:
                    b += (uint)(url[k + 7] << 24);
                    goto case 7;
                case 7:
                    b += (uint)(url[k + 6] << 16);
                    goto case 6;
                case 6:
                    b += (uint)(url[k + 5] << 8);
                    goto case 5;
                case 5:
                    b += (uint)(url[k + 4]);
                    goto case 4;
                case 4:
                    a += (uint)(url[k + 3] << 24);
                    goto case 3;
                case 3:
                    a += (uint)(url[k + 2] << 16);
                    goto case 2;
                case 2:
                    a += (uint)(url[k + 1] << 8);
                    goto case 1;
                case 1:
                    a += (uint)(url[k + 0]);
                    break;
                default:
                    break;
            }

            Mix(ref a, ref b, ref c);

            return string.Format("6{0}", c);
        }
    }
}
